from spacy import util
from spacy.tokens import Doc

from .CrossLingualPredictor import CrossLingualPredictor as Predictor


class CrossLingualPredictorSpacy(Predictor):
    def __init__(self, language: str, device: int = -1, model_name: str = 'info_xlm') -> None:
        super().__init__(language, device, model_name)
        Doc.set_extension("coref_clusters", default=None, force=True)
        Doc.set_extension("resolved_text", default=None, force=True)

    def __call__(self, doc: Doc) -> Doc:
        """
        predict the class for a spacy Doc

        Args:
            doc (Doc): a spacy doc

        Returns:
            Doc: spacy doc with ._.cats key-class proba-value dict
        """
        prediction = super(self.__class__, self).predict(doc.text.replace("\n", " "))
        doc = self.assign_prediction_to_doc(doc, prediction)
        return doc

    def pipe(self, stream, batch_size=128):
        """
        predict the class for a spacy Doc stream

        Args:
            stream (Doc): a spacy doc

        Returns:
            Doc: spacy doc with ._.cats key-class proba-value dict
        """
        for docs in util.minibatch(stream, size=batch_size):
            texts = [doc.text.replace("\n", " ") for doc in docs]
            
            pred_results = super(self.__class__, self).pipe(texts)
            
            for doc, pred_result in zip(docs, pred_results):
                doc = self.assign_prediction_to_doc(doc, pred_result)
                
                yield doc

    @staticmethod
    def assign_prediction_to_doc(doc: Doc, prediction: dict):
        doc._.coref_clusters = prediction['clusters']
        doc._.resolved_text = prediction['resolved_text']
        return doc
        
        


from spacy.language import Language

from .CrossLingualPredictor import CrossLingualPredictor as Predictor
from .CrossLingualPredictorSpacy import \
    CrossLingualPredictorSpacy as SpacyPredictor

__all__ = ["Predictor"]

@Language.factory(
    "xx_coref",
    default_config={
        "device": -1,
        "model_name": 'info_xlm'
    },
)
def make_crosslingual_coreference(
    nlp: Language,
    name: str,
    device: int,
    model_name: str,
):  
    return SpacyPredictor(
        language=nlp.path.name.split('-')[0],
        device=device,
        model_name=model_name,
    )




from crosslingual_coreference import Predictor

from .data import texts

predictor = Predictor(language='nl_core_news_sm')

print(predictor.pipe(texts))

predictor = Predictor(language='nl_core_news_sm', model_name='xlm_roberta')

print(predictor.pipe(texts))



text_en = """Do not forget about Momofuku Ando! He created instant noodles in Osaka. At that location, Nissin was founded. Many students survived by eating his noodles, but they don't even know him."""
text_nl = """Vergeet Momofuku Ando niet! Hij maakte instantnoedels in Osaka. Op die locatie werd Nissin opgericht. Veel studenten overleefden door zijn noedels te eten, maar ze kennen zijn naam niet eens."""
text_de = """Vergiss Momofuku Ando nicht! Er kreierte Instantnudeln in Osaka. An diesem Standort wurde Nissin gegründet. Viele Studenten überlebten, indem sie seine Nudeln aßen, aber sie kennen ihn nicht einmal."""
text_dk = """Glem ikke Momofuku Ando! Han skabte instant nudler i Osaka. På det sted blev Nissin grundlagt. Mange elever overlevede ved at spise hans nudler, men de kender ham ikke engang."""
text_fr = """N'oubliez pas Momofuku Ando ! Il a créé des nouilles instantanées à Osaka. À cet endroit, Nissin a été fondée. De nombreux étudiants ont survécu en mangeant ses nouilles, mais ils ne le connaissent même pas."""
text_es = """¡No te olvides de Momofuku Ando! Creó fideos instantáneos en Osaka. En ese lugar, se fundó Nissin. Muchos estudiantes sobrevivieron comiendo sus fideos, pero ni siquiera lo conocen."""
text_it = """Non dimenticare Momofuku Ando! Ha creato spaghetti istantanei a Osaka. In quel luogo è stata fondata Nissin. Molti studenti sono sopravvissuti mangiando i suoi noodles, ma non lo conoscono nemmeno."""

texts = [text_en, text_nl, text_de, text_dk, text_fr, text_es, text_it]



import crosslingual_coreference
import spacy

from .data import texts

nlp = spacy.load('nl_core_news_sm')
nlp.add_pipe('xx_coref')

for doc in nlp.pipe(texts):
    print(doc._.coref_clusters)
    print(doc._.resolved_text)

nlp.remove_pipe('xx_coref')
nlp.add_pipe('xx_coref', config={"model_name": "xlm_roberta"})

for doc in nlp.pipe(texts):
    print(doc._.coref_clusters)
    print(doc._.resolved_text)
